/**************************************************************************

    This file is part of desktop95, a desktop manager for fvwm95.
    Copyright (C) 1996, 1997 David Barth, Hector Peraza.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

**************************************************************************/

#ifndef __OXFOLDER_H
#define __OXFOLDER_H

#include <X11/Xlib.h>

#include <xclass/OXFrame.h>
#include <xclass/OXWidgets.h>
#include <xclass/OXListView.h>
#include <xclass/OString.h>
#include <xclass/OPicture.h>

#include "OXNewWidgets.h"


//----- sort types

#define SORT_BY_NAME  0
#define SORT_BY_TYPE  1
#define SORT_BY_SIZE  2
#define SORT_BY_DATE  3


extern OXPopupMenu *objectMenu;

//-------------------------------------------------------------------

class OXFolderIcon : public OXIcon {
public:
  OXFolderIcon(const OXWindow *p, const OPicture *pic, int w, int h,
               unsigned int options = VERTICAL_FRAME,
               unsigned long back = _whitePixel) :
    OXIcon(p, pic, w, h, options, back) { _active = False; _selpic = NULL; }
  virtual ~OXFolderIcon();

  void Activate(int a);

protected:
  virtual void _DoRedraw();

  OSelectedPicture *_selpic;
  int _active;
};

class OXFolderLabel : public OXLabel {
protected:
  static GC _defaultGC;
  static XFontStruct *_defaultFontStruct;

public:
  OXFolderLabel(const OXWindow *p, OString *text,
                GC norm = _defaultGC,
                XFontStruct *font = _defaultFontStruct,
                unsigned int options = VERTICAL_FRAME,
                unsigned long back = _whitePixel) :
    OXLabel(p, text, norm, font, options, back) { _active = False; }

  void Activate(int a);

  friend class OXDesktopClient;

protected:
  virtual void _DoRedraw();
     
  int _active;
};

//-------------------------------------------------------------------

class OXFolder : public OXCompositeFrame {
protected:
  static Cursor _defaultCursor;

public:
  OXFolder(const OXWindow *p, const OPicture *pic, OString *name,
           int type, unsigned long size, int w, int h,
           unsigned int options = VERTICAL_FRAME,
           unsigned long back = _whitePixel);
  virtual ~OXFolder();

  void Activate(int a);
  int IsActive() const { return _active; }
  const OString *GetName() const { return _label->GetText(); }
  int GetType() const { return _type; }
  int GetSize() const { return _size; }
  virtual void Layout();

//+++
  virtual int HandleButton(XButtonEvent *event);
  virtual int HandleDoubleClick(XButtonEvent *event);

  void SetFileWindow(OXFrame *f) { _fw = f; }

  friend class OXDesktopClient;

protected:
  OXFolderIcon *_icon;
  OXFolderLabel *_label;
  int _active;
  int _type;
  unsigned long _size;

  OXFrame *_fw;
};

//-------------------------------------------------------------------

class OXFolderContainer : public OXCompositeFrame {
protected:
  static GC _lineGC;

public:
  OXFolderContainer(const OXWindow *p, int w, int h,
                    unsigned int options = SUNKEN_FRAME,
                    unsigned long back = _defaultFrameBackground);

  virtual void AddFolder(OPicture *icon, OString *name, 
                         int type, unsigned long size, int small);
  virtual int HandleButton(XButtonEvent *event);
  virtual int HandleDoubleClick(XButtonEvent *event);
  virtual int HandleMotion(XMotionEvent *event);

  void UseStatusBar(OXStatusBar *stb) { _stb = stb; }
  void Sort(int sortType);
  void ChangeDirectory(OString *path);
  void DisplayDirectory();
  void RemoveAllFolders();

  friend class OXDesktopClient;

private:
  OPicture *FolderPic(char *f_name, int *type, unsigned long *size);
  SListFrameElt **_doSort(SListFrameElt **head, int n);
  int _compare(SListFrameElt *p1, SListFrameElt *p2);
  void CreateFolders();
  void DisplayTotal();

protected:
  OXStatusBar *_stb;
  OXFolder *_last_active;
  OXPopupMenu *_popup, *_subpopup;
  int _xp, _yp, _x0, _xf, _y0, _yf;
  int _total, _selected, _dragging;
  int _sortType;
};


#endif
