/**************************************************************************

    This file is part of desktop95, a desktop manager for fvwm95.
    Copyright (C) 1996, 1997 David Barth, Hector Peraza.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

**************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#include <xclass/OXClient.h>
#include <xclass/OXWindow.h>
#include <xclass/OXFrame.h>
#include <xclass/OXWidgets.h>
#include <xclass/OXMsgBox.h>
#include <xclass/OString.h>
#include <xclass/OPicture.h>
#include <xclass/utils.h>

#include "OXNewClient.h"
#include "OXNewWidgets.h"
#include "OXFolder.h"

#include "main.h"

//-------------------------------------------------------------------

OXDesktopClient *clientX;
OXDesktopMainFrame *mainWindow;

OXFolderContainer *fileWindow;
OXPopupMenu *objectMenu;
OXPopupMenu *sendToMenu;

char *AppName;
char *StartDir;

const OPicture *Folder_t;
const OPicture *ExeDoc_t;
const OPicture *TextDoc_t;
const OPicture *Folder_s;
const OPicture *ExeDoc_s;
const OPicture *TextDoc_s;

int SmallIcons = False;
int NewBrowser = True;


main(int argc, char *argv[]) {
  char *p;

  if ((p = strrchr(argv[0], '/')) == NULL)
    AppName = argv[0];
  else
    AppName = ++p;

  if (argc > 1)
    StartDir = argv[1];
  else
    StartDir = "/home/peraza/.desktop";

  clientX = new OXDesktopClient;

  Debug(DBG_INFO, "Creating main window...\n");
 
  mainWindow = new OXDesktopMainFrame(clientX->GetRoot(), 400, 200);

  Folder_s  = clientX->GetPicture("folder.s.xpm");
  ExeDoc_s  = clientX->GetPicture("app.s.xpm");
  TextDoc_s = clientX->GetPicture("doc.s.xpm");

  Folder_t  = clientX->GetPicture("folder.t.xpm");
  ExeDoc_t  = clientX->GetPicture("app.t.xpm");
  TextDoc_t = clientX->GetPicture("doc.t.xpm");

  if (!Folder_s || !Folder_t)
    FatalError("Missing pixmap(s) \"folder.*.xpm\"");
  if (!ExeDoc_s || !ExeDoc_t)
    FatalError("Missing pixmap(s) \"app.*.xpm\"");
  if (!TextDoc_s || !TextDoc_t)
    FatalError("Missing pixmap(s) \"doc.*.xpm\"");

  fileWindow->ChangeDirectory(new OString(StartDir));

  // we need to use GetDefault...() to initialize the layout algorithm...
//  mainWindow->Resize(mainWindow->GetDefaultSize());
//  mainWindow->Resize(400, 200);

  mainWindow->MapWindow();

  Debug(DBG_INFO, "Running...\n");
  clientX->Run();
}

OXDesktopMainFrame::OXDesktopMainFrame(const OXWindow *p, int w, int h) :
  OXMainFrame(p, w, h) {
  char tmp[BUFSIZ];

  //-------------- create menu bar and popup menus

  Debug(DBG_INFO, "Creating menus...\n");

  _menuBarLayout = new OLayoutHints(LHINTS_TOP | LHINTS_LEFT | LHINTS_EXPAND_X, 0, 0, 1, 1);
  _menuBarItemLayout = new OLayoutHints(LHINTS_TOP | LHINTS_LEFT, 0, 4, 0, 0);
  _menuBarHelpLayout = new OLayoutHints(LHINTS_TOP | LHINTS_RIGHT);

  sendToMenu = new OXPopupMenu(_client->GetRoot());
  sendToMenu->AddEntry(new OHotString("Fax Recipient"),  21, _client->GetPicture("fax.t.xpm"));
  sendToMenu->AddEntry(new OHotString("Mail Recipient"), 22, _client->GetPicture("mail.t.xpm"));
  sendToMenu->AddEntry(new OHotString("My Briefcase"),   23, _client->GetPicture("briefcase.t.xpm"));

  objectMenu = new OXPopupMenu(_client->GetRoot());
  objectMenu->AddEntry(new OHotString("&Open"),    1501);
  objectMenu->AddEntry(new OHotString("&Explore"), 1502);
  objectMenu->AddEntry(new OHotString("&Find..."), 1503);
  objectMenu->AddSeparator();
  objectMenu->AddPopup(new OHotString("Se&nd To"), sendToMenu);
  objectMenu->AddSeparator();
  objectMenu->AddEntry(new OHotString("Cu&t"),   M_EDIT_CUT);
  objectMenu->AddEntry(new OHotString("&Copy"),  M_EDIT_COPY);
  objectMenu->AddEntry(new OHotString("&Paste"), M_EDIT_PASTE);
  objectMenu->AddSeparator();
  objectMenu->AddEntry(new OHotString("Create &Shortcut"), M_FILE_NEWSHORTCUT);
  objectMenu->AddEntry(new OHotString("&Delete"),          M_FILE_DELETE);
  objectMenu->AddEntry(new OHotString("Rena&me"),          M_FILE_RENAME);
  objectMenu->AddSeparator();
  objectMenu->AddEntry(new OHotString("P&roperties"), M_FILE_PROPS);
  objectMenu->DefaultEntry(1501);

  sendToMenu->Associate(this);
  objectMenu->Associate(this);

  _newMenu = new OXPopupMenu(_client->GetRoot());
  _newMenu->AddEntry(new OHotString("&Folder"),       M_FILE_NEWFOLDER);
  _newMenu->AddEntry(new OHotString("&Shortcut"),     M_FILE_NEWSHORTCUT);
  _newMenu->AddSeparator();
  _newMenu->AddEntry(new OHotString("Bitmap Image"),  113);
  _newMenu->AddEntry(new OHotString("Text Document"), 113);
  _newMenu->AddEntry(new OHotString("Briefcase"),     113);

  _menuFile = new OXPopupMenu(_client->GetRoot());
  _menuFile->AddEntry(new OHotString("&Open"),     1501);
  _menuFile->AddEntry(new OHotString("&Explore"),  1502);
  _menuFile->AddEntry(new OHotString("&Find..."),  1503);
  _menuFile->AddSeparator();
  _menuFile->AddPopup(new OHotString("Se&nd To"),  sendToMenu);
  _menuFile->AddSeparator();
  _menuFile->AddPopup(new OHotString("Ne&w"), _newMenu);
  _menuFile->AddSeparator();
  _menuFile->AddEntry(new OHotString("Create &Shortcut"), M_FILE_NEWSHORTCUT);
  _menuFile->AddEntry(new OHotString("&Delete"),          M_FILE_DELETE);
  _menuFile->AddEntry(new OHotString("Rena&me"),          M_FILE_RENAME);
  _menuFile->AddEntry(new OHotString("P&roperties"),      M_FILE_PROPS);
  _menuFile->AddSeparator();
  _menuFile->AddEntry(new OHotString("&Close"),           M_FILE_CLOSE);
  _menuFile->DefaultEntry(1501);

  _newMenu->DisableEntry(M_FILE_NEWSHORTCUT);
  _menuFile->DisableEntry(M_FILE_NEWSHORTCUT);
  _menuFile->DisableEntry(M_FILE_DELETE);
  _menuFile->DisableEntry(M_FILE_RENAME);
  _menuFile->DisableEntry(M_FILE_PROPS);

  _menuEdit = new OXPopupMenu(_client->GetRoot());
  _menuEdit->AddEntry(new OHotString("&Undo"),             M_EDIT_UNDO);
  _menuEdit->AddSeparator();
  _menuEdit->AddEntry(new OHotString("Cu&t"),              M_EDIT_CUT);
  _menuEdit->AddEntry(new OHotString("&Copy"),             M_EDIT_COPY);
  _menuEdit->AddEntry(new OHotString("&Paste"),            M_EDIT_PASTE);
  _menuEdit->AddEntry(new OHotString("Paste &Shortcut"),   M_EDIT_PASTESHORTCUT);
  _menuEdit->AddSeparator();
  _menuEdit->AddEntry(new OHotString("Select &All"),       M_EDIT_SELECTALL);
  _menuEdit->AddEntry(new OHotString("&Invert Selection"), M_EDIT_INVSELECTION);
//_menuEdit->DefaultEntry(M_EDIT_INVSELECTION);

  _sortMenu = new OXPopupMenu(_client->GetRoot());
  _sortMenu->AddEntry(new OHotString("By &Name"),      M_VIEW_ARRANGE_BYNAME);
  _sortMenu->AddEntry(new OHotString("By &Type"),      M_VIEW_ARRANGE_BYTYPE);
  _sortMenu->AddEntry(new OHotString("By &Size"),      M_VIEW_ARRANGE_BYSIZE);
  _sortMenu->AddEntry(new OHotString("By &Date"),      M_VIEW_ARRANGE_BYDATE);
  _sortMenu->AddSeparator();
  _sortMenu->AddEntry(new OHotString("&Auto Arrange"), M_VIEW_ARRANGE_AUTO);
  _sortMenu->CheckEntry(M_VIEW_ARRANGE_AUTO);
  _sortMenu->RCheckEntry(M_VIEW_ARRANGE_BYNAME, 
                         M_VIEW_ARRANGE_BYNAME, M_VIEW_ARRANGE_BYDATE);

  _menuView = new OXPopupMenu(_client->GetRoot());
  _menuView->AddEntry(new OHotString("Status &Bar"),   M_VIEW_STATUS);
  _menuView->AddSeparator();
  _menuView->AddEntry(new OHotString("Lar&ge Icons"),  M_VIEW_LARGEICONS);
  _menuView->AddEntry(new OHotString("S&mall Icons"),  M_VIEW_SMALLICONS);
  _menuView->AddEntry(new OHotString("&List"),         M_VIEW_LIST);
  _menuView->AddEntry(new OHotString("&Details"),      M_VIEW_DETAILS);
  _menuView->AddSeparator();
  _menuView->AddPopup(new OHotString("Arrange &Icons"), _sortMenu);
  _menuView->AddEntry(new OHotString("Lin&e up Icons"), M_VIEW_LINEUP);
  _menuView->AddSeparator();
  _menuView->AddEntry(new OHotString("&Refresh"),      M_VIEW_REFRESH);

  _menuHelp = new OXPopupMenu(_client->GetRoot());
  _menuHelp->AddEntry(new OHotString("&Contents"),  M_HELP_CONTENTS);
  _menuHelp->AddEntry(new OHotString("&Search..."), M_HELP_SEARCH);
  _menuHelp->AddSeparator();
  _menuHelp->AddEntry(new OHotString("&About"),     M_HELP_ABOUT);

  _menuFile->Associate(this);
  _menuEdit->Associate(this);
  _menuView->Associate(this);
  _menuHelp->Associate(this);
  _sortMenu->Associate(this);

  Debug(DBG_INFO, "Creating menu bar...\n");
  _menuBar = new OXMenuBar(this, 1, 1, HORIZONTAL_FRAME);
  _menuBar->AddPopup(new OHotString("&File"), _menuFile, _menuBarItemLayout);
  _menuBar->AddPopup(new OHotString("&Edit"), _menuEdit, _menuBarItemLayout);
  _menuBar->AddPopup(new OHotString("&View"), _menuView, _menuBarItemLayout);
  _menuBar->AddPopup(new OHotString("&Help"), _menuHelp, _menuBarItemLayout);
//_menuBarHelpLayout);

  Debug(DBG_INFO, "Creating subframes...\n");
  AddFrame(_menuBar, _menuBarLayout);

  //-------------- files

  _canvasWindow = new OXCanvas(this, 500, 250);

  fileWindow = new OXFolderContainer(_canvasWindow->GetViewPort(),
                                     500, 250, HORIZONTAL_FRAME,
                                     _whitePixel);

  _canvasWindow->SetContainer(fileWindow);

  SmallIcons = False;
  _menuView->RCheckEntry(M_VIEW_LARGEICONS, M_VIEW_LARGEICONS, M_VIEW_DETAILS);

  AddFrame(_canvasWindow, new OLayoutHints(LHINTS_EXPAND_X | LHINTS_EXPAND_Y,
 		 	                   0, 0, 1, 0));

  //-------------- status bar

  _statusBar = new OXStatusBar(this, 60, 10);

  AddFrame(_statusBar, new OLayoutHints(LHINTS_BOTTOM | LHINTS_EXPAND_X,
		 		        0, 0, 3, 0));

  fileWindow->UseStatusBar(_statusBar);

  //--------------

  Debug(DBG_INFO, "Startup...\n");

  SetWindowName(getcwd(tmp, BUFSIZ));
  SetClassHints("Desktop", "Desktop");

  SetMWMHints(MWM_DECOR_ALL, MWM_FUNC_ALL, MWM_INPUT_MODELESS);

  MapSubwindows();

  Resize(GetDefaultSize());

  _menuView->CheckEntry(M_VIEW_STATUS);
}

OXDesktopMainFrame::~OXDesktopMainFrame() {
  delete _statusBar;
  delete fileWindow; 
  delete _canvasWindow; 
 
  delete _menuBarLayout;
  delete _menuBarItemLayout;
  delete _menuBarHelpLayout; 
 
  delete _menuFile;
  delete _menuEdit;
  delete _menuView;
  delete _menuHelp;
  delete _newMenu;
  delete _sortMenu;

  delete objectMenu;
  delete sendToMenu;
}

int OXDesktopMainFrame::ProcessMessage(long msg, long parm1, long parm2) {

  switch (GET_MSG(msg)) {

    case XC_COMMAND:    // buttons and menu commands
      switch(GET_SUBMSG(msg)) {

        // process menu and buttons together
        case CM_BUTTON:
        case CM_MENU:
          switch(parm1) {

            case M_FILE_CLOSE:
              delete this; // mainWindow;
              delete clientX;
              exit(0);
              break;

            case M_VIEW_STATUS:
              if (GetState(_statusBar)) {
                HideFrame(_statusBar);
                _menuView->UnCheckEntry(M_VIEW_STATUS);
              } else {
                ShowFrame(_statusBar);
                _menuView->CheckEntry(M_VIEW_STATUS);
              }
              break;

            // test only...
            case M_VIEW_LARGEICONS:
              if (SmallIcons) {
                SmallIcons = False;
                _menuView->RCheckEntry(M_VIEW_LARGEICONS,
                                       M_VIEW_LARGEICONS, M_VIEW_DETAILS);
                fileWindow->DisplayDirectory();
              }
              break;

            case M_VIEW_SMALLICONS:
              if (!SmallIcons) {
                SmallIcons = True;
                _menuView->RCheckEntry(M_VIEW_SMALLICONS,
                                       M_VIEW_LARGEICONS, M_VIEW_DETAILS);
                fileWindow->DisplayDirectory();
              }
              break;

            case M_VIEW_LIST:
            case M_VIEW_DETAILS:
              if (SmallIcons) {
                _menuView->RCheckEntry(M_VIEW_SMALLICONS,
                                       M_VIEW_LARGEICONS, M_VIEW_DETAILS);
              } else {
                _menuView->RCheckEntry(M_VIEW_LARGEICONS,
                                       M_VIEW_LARGEICONS, M_VIEW_DETAILS);
              }
              XBell(GetDisplay(), 0);
              break;

            case M_VIEW_ARRANGE_BYNAME:
              _sortMenu->RCheckEntry(M_VIEW_ARRANGE_BYNAME,
                                     M_VIEW_ARRANGE_BYNAME, M_VIEW_ARRANGE_BYDATE);
              fileWindow->Sort(SORT_BY_NAME);
              break;

            case M_VIEW_ARRANGE_BYTYPE:
              _sortMenu->RCheckEntry(M_VIEW_ARRANGE_BYTYPE,
                                     M_VIEW_ARRANGE_BYNAME, M_VIEW_ARRANGE_BYDATE);
              fileWindow->Sort(SORT_BY_TYPE);
              break;

            case M_VIEW_ARRANGE_BYSIZE:
              _sortMenu->RCheckEntry(M_VIEW_ARRANGE_BYSIZE,
                                     M_VIEW_ARRANGE_BYNAME, M_VIEW_ARRANGE_BYDATE);
              fileWindow->Sort(SORT_BY_SIZE);
              break;

            case M_VIEW_ARRANGE_BYDATE:
              _sortMenu->RCheckEntry(M_VIEW_ARRANGE_BYDATE,
                                     M_VIEW_ARRANGE_BYNAME, M_VIEW_ARRANGE_BYDATE);
              fileWindow->Sort(SORT_BY_DATE);
              break;

            case M_VIEW_REFRESH:
              fileWindow->DisplayDirectory();
              break;

            case 99:
              fileWindow->ChangeDirectory(new OString(".."));
              break;

            default:
              XBell(GetDisplay(), 0);
              break;

          } // switch(parm1)
          break;

        default:
          break;

      } // switch(GET_SUBMSG(msg))
      break;

    default:
      break;

  } // switch(GET_MSG(msg))

  return True;
}
