/**************************************************************************

    This file is part of notepad95, a text editor for fvwm95.
    Copyright (C) 1997, Harald Radke.                   

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

**************************************************************************/


#include "OText.h"

OText::OText()
{
 first = new line;
 current = first;
 first->len = 0;
 first->next = first->prev = NULL;
 first->buffer = NULL;
 isSaved = True;
 colCount = 0;
 rowCount = 1;
 current_row = 0;
}

OText::~OText()				
{
  Clear();
  delete first;
}

void OText::Clear()
{
 line *travel = first;
 while (travel->next != NULL)
  travel = travel->next;
 while (travel != first)
 {
  delete travel->buffer;
  travel = travel->prev;
  delete travel->next;
 }
 delete first->buffer;
 first->len = 0;
 current_row = 0;
 colCount = 0; 
 rowCount = 1;
 isSaved = True;
// delete first;
}

bool OText::Load(const char *fn)
{
  long n;
  FILE *fp;
  int  i, cnt,a;
  char buf[8000], c, *src;
  char  *dst;
  char *buffer;
  line *travel, *temp;
  char buf2[8000];
  travel = NULL;
  if ((fp = fopen(fn, "r")) == NULL) return False;
  i = 0;
  while (fgets(buf, 8000, fp) != NULL) {
    // Expand tabs
    src = buf;
    dst = buf2;
    cnt = 0;   
    while (c = *src++) {
      if (c == 0x0D || c == 0x0A)
        break;
      else if (c == 0x09)
        do
          *dst++ = ' ';
        while (((dst-buf2) & 0x7) && (cnt++ < 8000-1));
      else
        *dst++ = c;
      if (cnt++ >= 8000-1) break;
    }
    *dst = '\0';
    temp = new line;
    buffer = new char[strlen(buf2)];
    strncpy(buffer, buf2, strlen(buf2));
    temp->len = strlen(buf2);
    temp->buffer = buffer;
    temp->prev = NULL;
    temp->next = NULL;
    if (colCount < temp->len)
     colCount = temp->len;

    if (travel != NULL)
    {
     travel->next = temp;
     temp->prev = travel;
    travel = travel->next; 
    }
    else
    {
     travel = temp;
     delete first;
     first = travel;
     current = first;
    }
   ++i;
    for (a=0; a < 8000; a++)
    {
     buf[a] = '\0';
     buf2[a] = '\0';
    }

  }
  fclose(fp);

  // Remember the number of lines
  rowCount = i;
  isSaved = True;
  return True;
}

bool OText::Save(const char *fn)	
{
 char *buffer;
 line *travel=first;
 FILE *fp;
 if ((fp = fopen(fn, "w")) == NULL) return False;
 while (travel != NULL)
 {
  buffer = new char[travel->len+2];
  strncpy(buffer, travel->buffer, travel->len);
  buffer[travel->len] = '\n';
  buffer[travel->len+1] = '\0';
  if (fputs(buffer, fp) == EOF) 
  {
   delete buffer;
   fclose(fp);
   return False;
  }
  delete buffer;
  travel = travel->next;
 }
 isSaved = True;
   fclose(fp);

 return True;
}


					
bool OText::DelChar(Coord pos)    
{
 char *new_line;
 line *temp;
 SetCurrentRow(pos.row);
 if (pos.col > 0)
 {
  if (current->len > 0)
  {
   new_line = new char[current->len-1];
   strncpy(new_line, current->buffer, pos.col -1);
   strncpy(new_line+pos.col-1, (current->buffer)+pos.col, current->len-pos.col);
   delete current->buffer;
   current->buffer = new_line;
   current->len--;  
   isSaved = False;
   LongestLine();
   return False;
  }
 }
 else
 {
  if (current_row > 0)
  {
   new_line = new char[current->len + current->prev->len];
   strncpy (new_line, current->prev->buffer, current->prev->len);
   strncpy (new_line + current->prev->len, current->buffer, current->len);
   current->prev->len+= current->len;
   temp = current;
   SetCurrentRow(current_row - 1);
   if (temp->next != NULL)
    temp->next->prev = current;
   current->next = temp->next;
   delete temp->buffer;
   delete temp;
   delete current->buffer;
   current->buffer = new_line;
   isSaved = False;
   LongestLine();
   return True;
  }
 }
}

void OText::InsChar(Coord pos, char c)		
{
 char *new_line;
 SetCurrentRow(pos.row);
 new_line = new char[current->len+1];
 if (current->len > 0)
 {
  strncpy(new_line, current->buffer, pos.col);
  new_line[pos.col] = c;
  strncpy((new_line+1+pos.col), (current->buffer)+pos.col, current->len - pos.col);
 }
 else
  new_line[0] = c;
 delete current->buffer;
 current->buffer = new_line;
 current->len++;
 isSaved = False; 
 LongestLine();
}

char OText::GetChar(Coord pos)
{
 SetCurrentRow(pos.row); 
 if (current->len <= pos.col)
  return -1;
 else
  return current->buffer[pos.col];
}




bool OText::DelText(Coord start, Coord end)
{
 char *new_line;
 line *temp, *temp2;
 long temp_row;
 
 SetCurrentRow(start.row);
 new_line = new char[start.col]; 
 strncpy (new_line, current->buffer, start.col);
 temp = new line;
 temp->next = current->next;
 temp->prev = current->prev;
 temp->buffer = new_line;
 temp->len  = start.col;
 temp_row = current_row;
 SetCurrentRow(end.row);
 if ((current->len > 0) && (current->len != end.col+1))
 {
  new_line = new char[temp->len+(current->len-end.col-1)];
  strncpy (new_line, temp->buffer, temp->len);
  strncpy (new_line+temp->len, current->buffer+end.col+1, current->len-end.col-1);
  temp->len+= current->len-end.col-1;
  delete temp->buffer;
  temp->buffer = new_line;
 }
 SetCurrentRow(temp_row);
 delete current->buffer;
 current->buffer = temp->buffer;
 current->len = temp->len;
 delete temp;
 SetCurrentRow(end.row);
 while(current_row > temp_row)
  DelLine(current_row);
 SetCurrentRow(temp_row);
 isSaved = False;
 LongestLine();
}

bool OText::InsText(Coord ins_pos, OText *src,Coord start_src, Coord 
end_src) {
 char *buffer;

 line *temp, *travel;
 long count=1;
 long len;

 SetCurrentRow(ins_pos.row);
 temp = new line;
 temp->next = temp->prev = NULL;
 temp->buffer = new char[current->len];
 strncpy(temp->buffer, current->buffer, current->len);
 temp->len = current->len;
 delete current->buffer;
 len = src->GetLineLength(start_src.row)-start_src.col;
 if (start_src.row == end_src.row)
  len = end_src.col - start_src.col+1;
 current->buffer = new char[ins_pos.col+len]; 
 buffer = new char[len];
 src->GetLine(start_src, buffer, len); 
 strncpy (current->buffer, temp->buffer, ins_pos.col);
 strncpy (current->buffer+ins_pos.col, buffer, len);
 delete buffer;
 current->len = ins_pos.col+len;
 start_src.row++;
 start_src.col = 0;
 while (start_src.row <= end_src.row)
 {
  rowCount++;
  travel = new line;
  travel->next = current->next;
  current->next = travel;
  travel->prev = current;
  if (travel->next != NULL)
   travel->next->prev = travel;
  if (start_src.row == end_src.row)
   len = end_src.col+1;
  else
   len = src->GetLineLength(start_src.row);
  travel->buffer  = new char[len];
  src->GetLine(start_src, travel->buffer, len);
  travel->len = len;
  current = current->next;
  current_row++;
  start_src.row++;
 }
 buffer = new char[current->len+temp->len-ins_pos.col];
 strncpy (buffer, current->buffer, current->len);
 strncpy (buffer+current->len, temp->buffer+ins_pos.col, temp->len-ins_pos.col);
 delete current->buffer;
 current->buffer = buffer;
 current->len +=temp->len-ins_pos.col;
 LongestLine();
 isSaved = False;
}
 



bool OText::InsLine(Coord pos, char *buffer, long length)
{
 char *temp_buffer;
 if (SetCurrentRow(pos.row) == False)
 {
  current->next = new line;
  current->next->prev = current;
  current = current->next;
  current->len = 0;
  current->next = NULL;
  current->buffer = NULL;
  rowCount++;
  current_row++;
 }
 temp_buffer = new char[length + current->len];
 strncpy (temp_buffer, current->buffer, pos.col);
 strncpy (temp_buffer+pos.col, buffer, length);
 strncpy (temp_buffer+pos.col+length, current->buffer+pos.col, current->len - pos.col);
 delete current->buffer;
 current->buffer = temp_buffer;
 current->len+= length;
 isSaved = False;
 LongestLine();

}

void OText::DelLine(long row)
{
  line *temp;

 SetCurrentRow(row);
 if (row > 0)
 {
  temp = current;
  current = current->prev;
  delete temp->buffer;
  current->next = temp->next;
  if (temp->next != NULL)
   temp->next->prev = current;
  delete temp; 
  current_row--;
  rowCount--;
  LongestLine();
 }
 else
  if (current->next != NULL)
  {
   temp = current;
   current = current->next;
   delete temp->buffer;
   current->prev = NULL;
   first = current;
   delete temp;
   rowCount--;
   LongestLine();
  }
  else
  {
   delete current->buffer;
   current->len = 0;
   rowCount--;
   colCount = 0;
  }
}

void OText::GetLine(Coord pos, char *buf, long length=0)
{
 SetCurrentRow(pos.row);
 if (length == 0)
  length = current->len;
 strncpy(buf, current->buffer+pos.col, length);
}

void OText::BreakLine(Coord pos)		 
{
 char *new_line;
 line *temp;

 SetCurrentRow(pos.row);
 new_line = new char[current->len - pos.col];
 strncpy(new_line, (current->buffer)+pos.col,current->len - pos.col);
 temp = new line;
 temp->buffer = new_line;
 temp->len = current->len - pos.col;
 temp->prev = current;
 temp->next = current->next;
 if (current->next != NULL)
  current->next->prev = temp;   
 current->next = temp;
 new_line = new char[pos.col];
 strncpy(new_line, current->buffer, pos.col);
 delete current->buffer;
 current->buffer = new_line;
 current->len = pos.col;
 isSaved = False;
 rowCount++;
 LongestLine();
}
							
long OText::GetLineLength(long row)
{
 if (SetCurrentRow(row))
  return current->len; 
 else
  return -1;
}

bool OText::SetCurrentRow(long row)
{
 int count;
 
 if (row > current_row)
 {
  for (count = current_row; count < row; count++)
  {
   if (current->next == NULL)
   {
    current_row = count;
    return False;
   }
   current = current->next;
  }
 }   
 else
 {
  for (count = current_row; count > row; count--)
  {
   if (current->prev == NULL)
   {
    current_row = count;
    return False;
   }
   current = current->prev;
  }
 }
 current_row = row;
 return True;
}
 
bool OText::Search(Coord *foundPos, Coord start, char *searchString,  
			bool direction, bool caseSensitive)
{
 if (SetCurrentRow(start.row) == False)
  return False;
 if (direction) 
 {
  while(1)
  {
   foundPos->col =  DownSearchBM(current->buffer+start.col, 
				 searchString, current->len - start.col,
				 caseSensitive);   
   if (foundPos->col != -1)
   {
    foundPos->col += start.col;
    foundPos->row = current_row;
    return True;
   }
   if (SetCurrentRow(current_row+1) == False)
    break;
   start.col = 0;
  }  
 }
 else
 {
  while(1)
  {
   foundPos->col =  UpSearchBM(current->buffer, searchString, start.col,
				 caseSensitive);  
   if (foundPos->col != -1)
   { 
    foundPos->row = current_row;
    return True;
   }
   if (SetCurrentRow(current_row-1) == False)
    break;
   start.col = current->len;
  }  
 } 
 return False;
}

long OText::DownSearchBM(char *actualLine, char *searchPattern, long len, bool cS) {  
 char d[95];
 long i, j, k, m,n;
 
 m = strlen(searchPattern);
 n = len;
 if ((len <= 0) || (m > n))
  return -1;

 for (i = 0; i < 95; i++)
  d[i] = m;
 for (i = 0; i < m - 1; i++)
 {
  d[searchPattern[i]-32] = m - i - 1;
  if (cS == False)
   if ((searchPattern[i] >= 65) && (searchPattern[i] <= 90))
    d[searchPattern[i]] == m - i - 1;
   else
    if ((searchPattern[i] >= 97) && (searchPattern[i] <= 122))   
     d[searchPattern[i]-64] = m - i - 1;
 }
 i = m;
 do
 {
  j = m;
  k = i;
  do
  {
   k--;
   j--;
  } while ((j >= 0) && CharEqual(searchPattern[j],actualLine[k],cS));
  i = i + d[actualLine[i-1]-32];
 } while ((j >= 0) && (i <= n));
 if (j < 0)
  return i - m - d[searchPattern[m-1] - 32];
 else
  return -1;
}


long OText::UpSearchBM(char *actualLine, char *searchPattern, long len, 
bool cS) {
 int d[95];
 long i, j, k, m,n;

 m = strlen(searchPattern); 
 n = len;
 if ((len <= 0) || (m > n))
  return -1;
 for (i = 0; i < 95; i++)
  d[i] = m;
 for (i = m - 1; i > 0; i--)
 {
  d[searchPattern[i]-32] = i;
  if (cS== False)
   if ((searchPattern[i] >= 65) && (searchPattern[i] <= 90))
    d[searchPattern[i]] = i;
   else
    if ((searchPattern[i] >= 97) && (searchPattern[i] <= 122))   
     d[searchPattern[i]-64] = i;
 }
 i = n - m-1;
 do
 {
  j = -1;
  k = i;
  do
  {
   k++;
   j++;
  } while ((j < m) && CharEqual(searchPattern[j],actualLine[k], cS));
  i = i - d[actualLine[i+1]-32];
 } while ((j < m) && (i >= 0));
 if (j >= m)
  return  i + m + d[searchPattern[1] - 32];
 else
  return -1;
}

bool OText::CharEqual(char first, char second, bool cS)
{
 if (first == second)
  return True;
 
 if (cS == False)
 {
  if ((first >= 65) && (first <= 90))
   if (first+32 == second)
    return True;

  if ((first >= 97) && (first <= 122))
   if (first-32 == second)
    return True;
 }

  return False;
}


void OText::LongestLine()
{
 line *travel;
 travel = first;
 colCount = 0;
 while(travel != NULL)
 {
  if (travel->len > colCount)
   colCount = travel->len;
  travel = travel->next;
 }
}

