/**************************************************************************

    This file is part of notepad95, a text editor for fvwm95.
    Copyright (C) 1997, Harald Radke.                   

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

**************************************************************************/


#include<stdio.h>
#include<stdlib.h>
#include<string.h>

#ifndef False 
 #define False (1==0)
 #define True (1==1)
#endif


typedef struct coord_type
{
 int row;
 int col;
} Coord;

typedef struct line_type
{
 char *buffer;
 long len;
 line_type *prev;
 line_type *next;
} line;

class OText
{
  public:
        OText();
	~OText();				

	void Clear();
	bool Load(const char *fn);			
	bool Save(const char *fn);		
	bool IsSaved();

					
	bool DelChar(Coord pos);    
	void InsChar(Coord pos, char c);		
        char GetChar(Coord pos);

	bool DelText(Coord start, Coord end);
        bool InsText(Coord ins_pos, OText *src, Coord start_src, Coord end_src);

        void DelLine(long row);
        bool InsLine(Coord pos, char *buf, long length);
	void GetLine(Coord pos, char *buf, long length);
	void BreakLine(Coord pos);		 
							
	long RowCount();
	long ColCount();
	long GetLineLength(long row);
 
	bool Search(Coord *foundPos, Coord star, char *searchString, 
                        bool direction, bool caseSensitive); 

  protected:
	bool isSaved;
	line *first, *current;
        int current_row;
	long rowCount, colCount;

        bool SetCurrentRow(long row);
	void LongestLine();
        long UpSearchBM(char *actualLine, char *searchPattern, long len, 
				bool cS); 
        long DownSearchBM(char *actualLine, char *searchPattern, long len,
				bool cS); 
        bool CharEqual(char first, char second, bool cS);
 };

inline bool OText::IsSaved()
{
	return isSaved;
}

inline long OText::RowCount()
{
	return rowCount;
}

inline long OText::ColCount()
{
	return colCount;
}

