/**************************************************************************

    This file is part of notepad95, a text editor for fvwm95.
    Copyright (C) 1997, Harald Radke.                   

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

**************************************************************************/

#include <stdio.h>

#include "OXTextEdit.h"


OXTextEdit::OXTextEdit(OXWindow *parent, int x, int y,
                       unsigned int options, unsigned long back) :
  OXCompositeFrame(parent, x, y, options, back) {

    SetLayoutManager(new OHorizontalLayout(this));

    int background = _client->GetColorByName("white");
    SetBackgroundColor(background);
    _vsb = new OXVScrollBar(this, 10, 10, CHILD_FRAME);
    _hsb = new OXHScrollBar(this, 10, 10, CHILD_FRAME);
    _textCanvas = new OXTextFrame(this, 10, 10, CHILD_FRAME, background);
    AddFrame(_textCanvas, NULL);
    AddFrame(_vsb, NULL);
    AddFrame(_hsb, NULL);

}

OXTextEdit::~OXTextEdit() {
  delete _textCanvas;
  delete _vsb;
  delete _hsb;
}

bool OXTextEdit::LoadFile(char *fname) {
  int retc = _textCanvas->LoadFile(fname);
//  _vsb->SetRange(_textCanvas->GetLines(), _textCanvas->GetVisibleLines());
  if (retc) {
    _hsb->SetPosition(0);
    _vsb->SetPosition(0);
    Layout();
  }
  return retc;
}

bool OXTextEdit::SaveFile(char *fname) {
 return _textCanvas->SaveFile(fname);
}

bool OXTextEdit::IsSaved() {
 return _textCanvas->IsSaved();
}

void OXTextEdit::Clear() {
  _textCanvas->Clear();
  Layout();
}

bool OXTextEdit::Cut()
{
 if (_textCanvas->Cut() == False)
  return False;
 Layout();
 return True;
}

bool OXTextEdit::Paste()
{
 if (_textCanvas->Paste() == False)
  return False;
 Layout();
 return True;
}

bool OXTextEdit::Copy()
{
 if (_textCanvas->Copy() == False)
  return False;
 return True;
}

bool OXTextEdit::Delete()
{
 if (_textCanvas->Delete() == False)
  return False;
 Layout();
 return True;
}

bool OXTextEdit::SelectAll()
{
 if (_textCanvas->SelectAll() == False)
  return False;
 return True;
}

bool OXTextEdit::LineBreak()
{
 if (_textCanvas->LineBreak() == False)
  return False;
 Layout();
 return True;
}

bool OXTextEdit::Search(char *string, bool direction, bool caseSensitive)
{
 return _textCanvas->Search(string, direction, caseSensitive);
}

bool OXTextEdit::Goto(long line)
{
 return _textCanvas->Goto(line);
}

int OXTextEdit::ProcessMessage(long msg, long parm1, long parm2) {
  switch(GET_MSG(msg)) {
    case XC_VSCROLL:
      switch(GET_SUBMSG(msg)) {
        case SB_SLIDERTRACK:
        case SB_SLIDERPOS:
          _textCanvas->SetTopLine(parm1);

          break;
      }
      break;
     case XC_HSCROLL:
      switch(GET_SUBMSG(msg)) {
        case SB_SLIDERTRACK:
        case SB_SLIDERPOS:
         _textCanvas->SetTopCol(parm1);

          break;
      }
      break;
     
     case XC_TEXTEDIT:
       switch(GET_SUBMSG(msg)) {
         case TXT_CLICK:
           SendMessage(_parent, MK_MSG(XC_TEXTEDIT,TXT_CLICK), parm1,parm2);
           break; 
         case TXT_ISMARKED:
           SendMessage(_parent, MK_MSG(XC_TEXTEDIT,TXT_ISMARKED), parm1,0);
           break;

         case TXT_POSCHANGE:
           SendMessage(_parent, MK_MSG(XC_TEXTEDIT,TXT_POSCHANGE), parm1,
                parm2);
           break;
         case TXT_VSBCHANGE:
           VLayout();
           break;

         case TXT_HSBCHANGE:
           HLayout();
           break;
     
         case TXT_SBPOS:
           SetSBPos(parm1, parm2);
           break;
    
     }
     break;         
  }
  return True;
}

void OXTextEdit::Layout() {
 VLayout();
 HLayout();
}

void OXTextEdit::DrawBorder() {
  switch (_options & (SUNKEN_FRAME | RAISED_FRAME | DOUBLE_BORDER)) {
  case SUNKEN_FRAME | DOUBLE_BORDER:
    XDrawLine (GetDisplay(), _id, _shadowGC, 0, 0, _w-2, 0);
    XDrawLine (GetDisplay(), _id, _shadowGC, 0, 0, 0, _h-2);
    XDrawLine (GetDisplay(), _id, _blackGC, 1, 1, _w-3, 1);
    XDrawLine (GetDisplay(), _id, _blackGC, 1, 1, 1, _h-3);

    XDrawLine (GetDisplay(), _id, _hilightGC, 0, _h-1, _w-1, _h-1);
    XDrawLine (GetDisplay(), _id, _hilightGC, _w-1, _h-1, _w-1, 0);
    XDrawLine (GetDisplay(), _id, _bckgndGC,  1, _h-2, _w-2, _h-2);
    XDrawLine (GetDisplay(), _id, _bckgndGC,  _w-2, 1, _w-2, _h-2);
    break;

  default:
    OXCompositeFrame::DrawBorder();
    break;
  }
}

void OXTextEdit::SetSBPos(int parm1, int parm2)
{
 _hsb->SetPosition(parm2);
 _vsb->SetPosition(parm1);

}

void OXTextEdit::HLayout() {
  int temp1, temp2;
  int tcw, tch;
  int cols, vcols;
  tch = _h - (_bw << 1);
  tcw = _w - (_bw << 1);
  if (_vsb->IsMapped() == True)
   tcw -= _vsb->GetDefaultWidth(); 
  _textCanvas->SetHeight(tch);
  _textCanvas->SetWidth(tcw);
  cols = _textCanvas->GetCols();
  vcols = _textCanvas->GetVisibleCols();

   if (cols <= vcols) 
   {
     if (_hsb->IsMapped() == True)
     {
      _textCanvas->SetTopCol(0);
      _hsb->UnmapWindow();
      VLayout();
     }
     else
      _textCanvas->MoveResize(_bw, _bw, tcw, tch); 

   } 
   else
   {
     tch -= _hsb->GetDefaultHeight();
     _textCanvas->MoveResize(_bw, _bw, tcw, tch);
     _hsb->MoveResize(_bw, _bw+tch, tcw,_hsb->GetDefaultHeight());
     _hsb->MapWindow();
    _hsb->SetRange(cols+1, vcols);
   }

}

void OXTextEdit::VLayout() {
  int lines, vlines, tcw, tch;
  tch = _h - (_bw << 1);
  tcw = _w - (_bw << 1);
  if (_hsb->IsMapped() == True)
   tch -= _hsb->GetDefaultHeight(); 
  _textCanvas->SetHeight(tch);
  _textCanvas->SetWidth(tcw);
  lines = _textCanvas->GetLines();
  vlines = _textCanvas->GetVisibleLines();

   if (lines <= vlines) 
   {
    if (_vsb->IsMapped() == True)
    {   
      _textCanvas->SetTopLine(0);
     _vsb->UnmapWindow();
      HLayout();
    }
     _textCanvas->MoveResize(_bw, _bw, tcw, tch); 

   } 
   else 
   {
     tcw -= _vsb->GetDefaultWidth();
     _textCanvas->MoveResize(_bw, _bw, tcw, tch); 
     _vsb->MoveResize(_bw + tcw, _bw, _vsb->GetDefaultWidth(), tch);
     _vsb->MapWindow();
    _vsb->SetRange(lines, vlines);
   }
}

