/**************************************************************************

    This file is part of Xclass95, a Win95-looking GUI toolkit.
    Copyright (C) 1996, 1997 David Barth, Hector Peraza.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

**************************************************************************/

#ifndef __OXCLIENT_H
#define __OXCLIENT_H

#include <X11/Xlib.h>
#include <xclass/OXSList.h>

extern Atom WM_DELETE_WINDOW;
extern Atom _MOTIF_WM_HINTS;
extern Atom _XCLASS_MESSAGE;

// Forward declarations to avoid include files headaches :-(

class OXWindow;
class OPicturePool;
class OPicture;
class OTimer;
class OXSTimerList;

// An OXClient is an exception in that is not a real
// OX object (no XID member)

// --------------------------------------------------------
// X-Window client

class OXClient {
public:
  OXClient(const char *DpyName = "");
  ~OXClient();

  Display *GetDisplay() const { return _dpy; }
  const OXWindow *GetRoot() const { return _Root; }
  OXWindow *GetWindowById(Window sw) const;

  unsigned long GetColorByName(const char *name) const;
  unsigned long GetHilite(const unsigned long base_color) const;
  unsigned long GetShadow(const unsigned long base_color) const;
  void NeedRedraw(OXWindow *w);
  int RegisterWindow(OXWindow *w);
  int UnregisterWindow(OXWindow *w);
  int RegisterTimer(OTimer *w);
  int UnregisterTimer(OTimer *w);
  void Run();
  void WaitFor(OXWindow *w);
  void WaitForUnmap(OXWindow *w);

  const OPicture *GetPicture(const char *name);
  const OPicture *GetPicture(const char *name, int new_width, int new_height);
  void FreePicture(const OPicture *pic);

protected:
  void MainLoop();
  int  ProcessOneEvent(int EventType, Window wid);
  int  HandleEvent(XEvent *Event);
  int  HandleMaskEvent(XEvent *Event, Window wid);
  int  _DoRedraw();

  unsigned long _backcolor, _forecolor, _hilite, _shadow;
  unsigned long _selbackcolor, _selforecolor, _white, _black;
  OXWindow *_Root;
  Display *_dpy;
  int _x_fd;
  OPicturePool *_picturePool;
  int _globalNeedRedraw;        // at least one window needs to be redrawn
  OXSList *_wlist;              // list of frames
  OXSTimerList *_tlist;         // list of timers
};


#endif  // __OXCLIENT_H
